//  : https://www.mql5.com/ru/forum/287618/page14#comment_9806429

#define BENCH(A)                                                              \
{                                                                             \
  const ulong StartTime = GetMicrosecondCount();                              \
  A;                                                                          \
  Print("Time[" + #A + "] = " + (string)(GetMicrosecondCount() - StartTime)); \
}

//   
int InitData( MqlTick &Ticks[], const int Amount = 1e7, const double Price = 1.2345)
{
  const int Size = ArrayResize(Ticks, Amount);

  for (int i = 0; i < Size; i++)
    Ticks[i].bid = Price;

  return(Size);
}

MqlTick TicksIn[];                  //    .
const int Init = InitData(TicksIn); //  .

// ,    .
bool IsCompare( const MqlTick &Ticks1[], const MqlTick &Ticks2[] )
{
  const int Size = ArraySize(Ticks1);
  bool Res = (Size == ArraySize(Ticks2));

  for (int i = 0; (i < Size) && Res; i++)
    Res = (Ticks1[i].bid == Ticks2[i].bid);

  return(Res);
}

typedef int (*T_TicksToIntArray)( const MqlTick &[], int &[] );
typedef int (*T_IntArrayToTicks)( const int &[], MqlTick &[] );

void Bench( T_TicksToIntArray TicksToIntArray, const string Str1, T_IntArrayToTicks IntArrayToTicks, const string Str2, const string Str = NULL )
{
  MqlTick TicksOut[];
  int Array[];

  Print("\n" + Str);

  Print(Str1);
  BENCH(TicksToIntArray(TicksIn, Array));  //   MqlTick[] -> int[].

  Print(Str2);
  BENCH(IntArrayToTicks(Array, TicksOut)); //   int[] -> MqlTick[].

  Print(IsCompare(TicksIn, TicksOut)); // ,       .
}

#define MACRO(A) A, #A

void OnStart()
{
//  Bench(MACRO(TicksToIntArray_fxsaber1), MACRO(IntArrayToTicks_fxsaber1), "https://www.mql5.com/ru/forum/287618/page14#comment_9806429");
  Bench(MACRO(TicksToIntArray_fxsaber2), MACRO(IntArrayToTicks_fxsaber2), "https://www.mql5.com/ru/forum/287618/page14#comment_9807465");
  Bench(MACRO(TicksToIntArray_antfx1), MACRO(IntArrayToTicks_antfx1), "https://www.mql5.com/ru/forum/287618/page14#comment_9808274");
}

//  :

/***********************************************/
// https://www.mql5.com/ru/forum/287618/page14#comment_9806429
#include <TypeToBytes.mqh> // https://www.mql5.com/ru/code/16280

template <typename T1, typename T2>
int ArrayToArray( const T1 &Source[], T2 &Target[] )
{
  return(_ArrayCopy(Target, Source) / sizeof(T2));
}

//      int[].
int TicksToIntArray_fxsaber1( const MqlTick &Ticks[], int &Array[] )
{
  return(ArrayToArray(Ticks, Array));
}

//   int[]   .
int IntArrayToTicks_fxsaber1( const int &Array[], MqlTick &Ticks[] )
{
  return(ArrayToArray(Array, Ticks));
}

/***********************************************/
// https://www.mql5.com/ru/forum/287618/page14#comment_9807465
template <typename T>
union INTEGER
{
  T Data;
  int Integer[sizeof(T) / sizeof(int)];
};

//      int[].
int TicksToIntArray_fxsaber2( const MqlTick &Ticks[], int &Array[] )
{
  INTEGER<MqlTick> TickInteger;

  const int Size1 = ArraySize(Ticks);
  const int Size2 = ArrayResize(Array, Size1 * sizeof(MqlTick) / sizeof(int));
  int j = 0;

  for (int i = 0; (i < Size1) && (j < Size2); i++)
  {
    TickInteger.Data = Ticks[i];

    j += ArrayCopy(Array, TickInteger.Integer, j);
  }

  return(j);
}

//   int[]   .
int IntArrayToTicks_fxsaber2( const int &Array[], MqlTick &Ticks[] )
{
  INTEGER<MqlTick> TickInteger;

  const int Size1 = ArraySize(Array);
  const int Size2 = ArrayResize(Ticks, Size1 * sizeof(int) / sizeof(MqlTick));
  int j = 0;

  for (int i = 0; (i < Size1) && (j < Size2); j++)
  {
    i += ArrayCopy(TickInteger.Integer, Array, 0, i, sizeof(MqlTick) / sizeof(int));

    Ticks[j] = TickInteger.Data;
  }

  return(j);
}

/***********************************************/
// https://www.mql5.com/ru/forum/287618/page14#comment_9808274

template< typename S, typename T >
union UDT{
  UDT(){for(int i=0;i<sizeof(S)/sizeof(T);i++)t[i]=0;}
  UDT(const S&src){s=src;}
  UDT(const T&src[],int k=0){for(int i=0;i<sizeof(S)/sizeof(T);i++)t[i]=src[i+k];}
  void to(S&dst){dst=s;}
  void to(T&dst[],int k=0){for(int i=0;i<sizeof(S)/sizeof(T);i++)dst[i+k]=t[i];}
  S s; T t[sizeof(S)/sizeof(T)];};

template< typename S, typename T >
int StructToArray( const S&src[],T&dst[]){
  ArrayResize(dst,ArraySize(src)*(sizeof(S)/sizeof(T)));
  for(int i=0;i<ArraySize(dst)/(sizeof(S)/sizeof(T));i++){UDT<S,T>u(src[i]);u.to(dst,i*(sizeof(S)/sizeof(T)));} return(ArraySize(dst));}

template< typename S, typename T >
int ArrayToStruct( const T&src[],S&dst[]){
  ArrayResize(dst,ArraySize(src)/(sizeof(S)/sizeof(T)));
  for(int i=0;i<ArraySize(dst);i++){UDT<S,T>u(src,i*(sizeof(S)/sizeof(T)));u.to(dst[i]);} return(ArraySize(dst));}

//      int[].
int TicksToIntArray_antfx1( const MqlTick &Ticks[], int &Array[] )
{
  return(StructToArray(Ticks, Array));
}

//   int[]   .
int IntArrayToTicks_antfx1( const int &Array[], MqlTick &Ticks[] )
{
  return(ArrayToStruct(Array, Ticks));
}

